@@ -17,7 +17,7 @@ VOICE_LANGUAGES = ('en', 'en-gb', 'es', 'fr', 'it', 'de', 'da-DK', 'de-DE',
                    'pt-PT', 'ru-RU', 'sv-SE', 'zh-CN', 'zh-HK', 'zh-TW')
 
 
-class Twilio(object):
+class Twilio:
     """
     Gateway for sending text messages and making phone calls using Twilio_.
 
@@ -34,6 +34,11 @@ class Twilio(object):
       Should be set to a verified phone number. Twilio_ differentiates between
       numbers verified for making phone calls and sending text messages.
 
+    ``TWILIO_MESSAGING_SERVICE_SID``
+      Can be set to a Twilio Messaging Service for SMS. This service can wrap multiple
+      phone numbers and choose one depending on the destination country.
+      When left empty the ``TWILIO_CALLER_ID`` will be used as sender ID.
+
     .. _Twilio: http://www.twilio.com/
     """
 
@@ -61,10 +66,17 @@ class Twilio(object):
             'two_factor/twilio/sms_message.html',
             {'token': token}
         )
-        self.client.messages.create(
-            to=device.number.as_e164,
-            from_=getattr(settings, 'TWILIO_CALLER_ID'),
-            body=body)
+        send_kwargs = {
+            'to': device.number.as_e164,
+            'body': body
+        }
+        messaging_service_sid = getattr(settings, 'TWILIO_MESSAGING_SERVICE_SID', None)
+        if messaging_service_sid is not None:
+            send_kwargs['messaging_service_sid'] = messaging_service_sid
+        else:
+            send_kwargs['from_'] = getattr(settings, 'TWILIO_CALLER_ID')
+
+        self.client.messages.create(**send_kwargs)
 
 
 def validate_voice_locale(locale):
