@@ -12,7 +12,6 @@ from django.conf import settings
 from django.contrib.auth import REDIRECT_FIELD_NAME, login
 from django.contrib.auth.decorators import login_required
 from django.contrib.auth.forms import AuthenticationForm
-from django.contrib.auth.views import SuccessURLAllowedHostsMixin
 from django.contrib.sites.shortcuts import get_current_site
 from django.core.signing import BadSignature
 from django.forms import Form, ValidationError
@@ -26,7 +25,7 @@ from django.utils.translation import gettext as _
 from django.views.decorators.cache import never_cache
 from django.views.decorators.csrf import csrf_protect
 from django.views.decorators.debug import sensitive_post_parameters
-from django.views.generic import DeleteView, FormView, TemplateView
+from django.views.generic import FormView, TemplateView
 from django.views.generic.base import View
 from django_otp import devices_for_user
 from django_otp.decorators import otp_required
@@ -34,15 +33,17 @@ from django_otp.plugins.otp_static.models import StaticDevice, StaticToken
 from django_otp.util import random_hex
 
 from two_factor import signals
-from two_factor.models import get_available_methods
+from two_factor.plugins.phonenumber.utils import (
+    backup_phones, get_available_phone_methods,
+)
+from two_factor.plugins.registry import registry
 from two_factor.utils import totp_digits
 
 from ..forms import (
     AuthenticationTokenForm, BackupTokenForm, DeviceValidationForm, MethodForm,
-    PhoneNumberForm, PhoneNumberMethodForm, TOTPDeviceForm, YubiKeyDeviceForm,
+    TOTPDeviceForm,
 )
-from ..models import PhoneDevice, get_available_phone_methods
-from ..utils import backup_phones, default_device, get_otpauth_url
+from ..utils import default_device, get_otpauth_url
 from .utils import (
     IdempotentSessionWizardView, class_view_decorator,
     get_remember_device_cookie, validate_remember_device_cookie,
@@ -50,25 +51,24 @@ from .utils import (
 
 try:
     from django.utils.http import url_has_allowed_host_and_scheme
-except ImportError:
+except ImportError:  # django<3.0
     from django.utils.http import (
         is_safe_url as url_has_allowed_host_and_scheme,
     )
 
 try:
-    from otp_yubikey.models import RemoteYubikeyDevice, ValidationService
-except ImportError:
-    ValidationService = RemoteYubikeyDevice = None
-
+    from django.contrib.auth.views import RedirectURLMixin
+except ImportError:  # django<4.1
+    from django.contrib.auth.views import (
+        SuccessURLAllowedHostsMixin as RedirectURLMixin,
+    )
 
 logger = logging.getLogger(__name__)
 
 REMEMBER_COOKIE_PREFIX = getattr(settings, 'TWO_FACTOR_REMEMBER_COOKIE_PREFIX', 'remember-cookie_')
 
 
-@class_view_decorator(sensitive_post_parameters())
-@class_view_decorator(never_cache)
-class LoginView(SuccessURLAllowedHostsMixin, IdempotentSessionWizardView):
+class LoginView(RedirectURLMixin, IdempotentSessionWizardView):
     """
     View for handling the login process, including OTP verification.
 
@@ -85,10 +85,6 @@ class LoginView(SuccessURLAllowedHostsMixin, IdempotentSessionWizardView):
         ('token', AuthenticationTokenForm),
         ('backup', BackupTokenForm),
     )
-    idempotent_dict = {
-        'token': False,
-        'backup': False,
-    }
     redirect_authenticated_user = False
     storage_name = 'two_factor.views.utils.LoginStorage'
 
@@ -262,11 +258,15 @@ class LoginView(SuccessURLAllowedHostsMixin, IdempotentSessionWizardView):
             return {}
         return super().process_step_files(form)
 
-    def get_form(self, *args, **kwargs):
+    def get_form(self, step=None, **kwargs):
         """
         Returns the form for the step
         """
-        form = super().get_form(*args, **kwargs)
+        if (step or self.steps.current) == 'token':
+            # Set form class dynamically depending on user device.
+            method = registry.method_from_device(self.get_device())
+            self.form_list['token'] = method.get_token_form_class()
+        form = super().get_form(step=step, **kwargs)
         if self.show_timeout_error:
             form.cleaned_data = getattr(form, 'cleaned_data', {})
             form.add_error(None, ValidationError(_('Your session has timed out. Please login again.')))
@@ -298,7 +298,9 @@ class LoginView(SuccessURLAllowedHostsMixin, IdempotentSessionWizardView):
         either making a phone call or sending a text message.
         """
         if self.steps.current == 'token':
-            self.get_device().generate_challenge()
+            form_with_errors = form and form.is_bound and not form.is_valid()
+            if not form_with_errors:
+                self.get_device().generate_challenge()
         return super().render(form, **kwargs)
 
     def get_user(self):
@@ -412,26 +414,13 @@ class SetupView(IdempotentSessionWizardView):
     form_list = (
         ('welcome', Form),
         ('method', MethodForm),
-        ('generator', TOTPDeviceForm),
-        ('sms', PhoneNumberForm),
-        ('call', PhoneNumberForm),
-        ('validation', DeviceValidationForm),
-        ('yubikey', YubiKeyDeviceForm),
+        # Other forms are dynamically added in get_form_list()
     )
-    condition_dict = {
-        'generator': lambda self: self.get_method() == 'generator',
-        'call': lambda self: self.get_method() == 'call',
-        'sms': lambda self: self.get_method() == 'sms',
-        'validation': lambda self: self.get_method() in ('sms', 'call'),
-        'yubikey': lambda self: self.get_method() == 'yubikey',
-    }
-    idempotent_dict = {
-        'yubikey': False,
-    }
 
     def get_method(self):
         method_data = self.storage.validated_step_data.get('method', {})
-        return method_data.get('method', None)
+        method_key = method_data.get('method', None)
+        return registry.get_method(method_key)
 
     def get(self, request, *args, **kwargs):
         """
@@ -441,16 +430,31 @@ class SetupView(IdempotentSessionWizardView):
             return redirect(self.success_url)
         return super().get(request, *args, **kwargs)
 
+    def get_form(self, step=None, **kwargs):
+        # Until https://github.com/jazzband/django-formtools/pull/62 is merged
+        if (step or self.steps.current) not in self.form_list:
+            self.form_list = self.get_form_list()
+        return super().get_form(step=step, **kwargs)
+
     def get_form_list(self):
         """
-        Check if there is only one method, then skip the MethodForm from form_list
+        Check if there is only one method, then skip the MethodForm from form_list.
         """
         form_list = super().get_form_list()
-        available_methods = get_available_methods()
+
+        available_methods = registry.get_methods()
         if len(available_methods) == 1:
             form_list.pop('method', None)
-            method_key, _ = available_methods[0]
+            method_key = available_methods[0].code
             self.storage.validated_step_data['method'] = {'method': method_key}
+        method = self.get_method()
+        if method:
+            form_list.update(method.get_setup_forms(self))
+        else:
+            for method in available_methods:
+                form_list.update(method.get_setup_forms(self))
+        if {'sms', 'call'} & set(form_list.keys()):
+            form_list['validation'] = DeviceValidationForm
         return form_list
 
     def render_next_step(self, form, **kwargs):
@@ -477,18 +481,18 @@ class SetupView(IdempotentSessionWizardView):
         except KeyError:
             pass
 
+        method = self.get_method()
         # TOTPDeviceForm
-        if self.get_method() == 'generator':
+        if method.code == 'generator':
             form = [form for form in form_list if isinstance(form, TOTPDeviceForm)][0]
             device = form.save()
-
-        # PhoneNumberForm / YubiKeyDeviceForm
-        elif self.get_method() in ('call', 'sms', 'yubikey'):
+        # PhoneNumberForm / YubiKeyDeviceForm / EmailForm
+        elif method.code in ('call', 'sms', 'yubikey', 'email'):
             device = self.get_device()
             device.save()
 
         else:
-            raise NotImplementedError("Unknown method '%s'" % self.get_method())
+            raise NotImplementedError("Unknown method '%s'" % method.code)
 
         django_otp.login(self.request, device)
         return redirect(self.success_url)
@@ -517,27 +521,12 @@ class SetupView(IdempotentSessionWizardView):
 
         Only used for call / sms -- generator uses other procedure.
         """
-        method = self.get_method()
-        kwargs = kwargs or {}
-        kwargs['name'] = 'default'
-        kwargs['user'] = self.request.user
-
-        if method in ('call', 'sms'):
-            kwargs['method'] = method
-            kwargs['number'] = self.storage.validated_step_data\
-                .get(method, {}).get('number')
-            return PhoneDevice(key=self.get_key(method), **kwargs)
-
-        if method == 'yubikey':
-            kwargs['public_id'] = self.storage.validated_step_data\
-                .get('yubikey', {}).get('token', '')[:-32]
-            try:
-                kwargs['service'] = ValidationService.objects.get(name='default')
-            except ValidationService.DoesNotExist:
-                raise KeyError("No ValidationService found with name 'default'")
-            except ValidationService.MultipleObjectsReturned:
-                raise KeyError("Multiple ValidationService found with name 'default'")
-            return RemoteYubikeyDevice(**kwargs)
+        if not getattr(self, '_device', None):
+            method = self.get_method()
+            self._device = method.get_device_from_setup_data(
+                self.request, self.storage.validated_step_data, key=self.get_key(method.code)
+            )
+        return self._device
 
     def get_key(self, step):
         self.storage.extra_data.setdefault('keys', {})
@@ -555,7 +544,8 @@ class SetupView(IdempotentSessionWizardView):
             b32key = b32encode(rawkey).decode('utf-8')
             self.request.session[self.session_key_name] = b32key
             context.update({
-                'QR_URL': reverse(self.qrcode_url)
+                'QR_URL': reverse(self.qrcode_url),
+                'secret_key': b32key,
             })
         elif self.steps.current == 'validation':
             context['device'] = self.get_device()
@@ -609,94 +599,6 @@ class BackupTokensView(FormView):
         return redirect(self.success_url)
 
 
-@class_view_decorator(never_cache)
-@class_view_decorator(otp_required)
-class PhoneSetupView(IdempotentSessionWizardView):
-    """
-    View for configuring a phone number for receiving tokens.
-
-    A user can have multiple backup :class:`~two_factor.models.PhoneDevice`
-    for receiving OTP tokens. If the primary phone number is not available, as
-    the battery might have drained or the phone is lost, these backup phone
-    numbers can be used for verification.
-    """
-    template_name = 'two_factor/core/phone_register.html'
-    success_url = settings.LOGIN_REDIRECT_URL
-    form_list = (
-        ('setup', PhoneNumberMethodForm),
-        ('validation', DeviceValidationForm),
-    )
-    key_name = 'key'
-
-    def get(self, request, *args, **kwargs):
-        """
-        Start the setup wizard. Redirect if no phone methods available.
-        """
-        if not get_available_phone_methods():
-            return redirect(self.success_url)
-        return super().get(request, *args, **kwargs)
-
-    def done(self, form_list, **kwargs):
-        """
-        Store the device and redirect to profile page.
-        """
-        self.get_device(user=self.request.user, name='backup').save()
-        return redirect(self.success_url)
-
-    def render_next_step(self, form, **kwargs):
-        """
-        In the validation step, ask the device to generate a challenge.
-        """
-        next_step = self.steps.next
-        if next_step == 'validation':
-            self.get_device().generate_challenge()
-        return super().render_next_step(form, **kwargs)
-
-    def get_form_kwargs(self, step=None):
-        """
-        Provide the device to the DeviceValidationForm.
-        """
-        if step == 'validation':
-            return {'device': self.get_device()}
-        return {}
-
-    def get_device(self, **kwargs):
-        """
-        Uses the data from the setup step and generated key to recreate device.
-        """
-        kwargs = kwargs or {}
-        kwargs.update(self.storage.validated_step_data.get('setup', {}))
-        return PhoneDevice(key=self.get_key(), **kwargs)
-
-    def get_key(self):
-        """
-        The key is preserved between steps and stored as ascii in the session.
-        """
-        if self.key_name not in self.storage.extra_data:
-            key = random_hex(20)
-            self.storage.extra_data[self.key_name] = key
-        return self.storage.extra_data[self.key_name]
-
-    def get_context_data(self, form, **kwargs):
-        kwargs.setdefault('cancel_url', resolve_url(self.success_url))
-        return super().get_context_data(form, **kwargs)
-
-
-@class_view_decorator(never_cache)
-@class_view_decorator(otp_required)
-class PhoneDeleteView(DeleteView):
-    """
-    View for removing a phone number used for verification.
-    """
-    success_url = settings.LOGIN_REDIRECT_URL
-
-    def get_queryset(self):
-        return self.request.user.phonedevice_set.filter(name='backup')
-
-    def get_success_url(self):
-        return resolve_url(self.success_url)
-
-
 @class_view_decorator(never_cache)
 @class_view_decorator(otp_required)
 class SetupCompleteView(TemplateView):
